@@ -6,6 +6,7 @@ represented by a list of dictionaries.
 
 from .request import Request
 from .exceptions import ChrisRequestException
+import json
 
 
 class Client(object):
@@ -13,7 +14,7 @@ class Client(object):
     A ChRIS API client.
     """
 
-    def __init__(self, url, username, password, timeout=30):
+    def __init__(self, url, username=None, password=None, timeout=30):
         self.url = url
         self.query_url_sufix = 'search/'
         self.username = username
@@ -51,7 +52,6 @@ class Client(object):
         # get urls of the high level API resources
         self.chris_instance_url = self.chris_instance_url or get_url(
             coll, 'chrisinstance')[0]
-        self.admin_url = self.admin_url or get_url(coll, 'admin')[0]
         self.files_url = self.files_url or get_url(coll, 'files')[0]
         self.compute_resources_url = self.compute_resources_url or get_url(
             coll, 'compute_resources')[0]
@@ -70,7 +70,12 @@ class Client(object):
         self.service_files_url = self.service_files_url or get_url(
             coll, 'servicefiles')[0]
         self.file_browser_url = self.file_browser_url or get_url(coll, 'filebrowser')[0]
-        self.user_url = self.user_url or get_url(coll, 'user')[0]
+        if self.username:
+            self.user_url = self.user_url or get_url(coll, 'user')[0]
+            if not self.admin_url:
+                urls = get_url(coll, 'admin')
+                if urls:
+                    self.admin_url = urls[0]
 
     def get_feeds(self, search_params=None):
         """
@@ -116,20 +121,27 @@ class Client(object):
             return Request.get_data_from_collection(coll)
         return {'data': [], 'hasNextPage': False, 'hasPreviousPage': False, 'total': 0}
 
-    def admin_upload_plugin(self, compute_names, fname):
+    def admin_upload_plugin(self, compute_names, data):
         """
-        Upload a plugin representation file and create a new plugin. The fname argument
-        can be a string indicating a local file path or a file handler.
+        Upload a plugin representation file and create a new plugin. The data argument
+        can be:
+            * a string indicating a local file path, or
+            * a file handler, or
+            * a python dictionary representation.
         """
         if not self.admin_url: self.set_urls()
-        if isinstance(fname, str):
-            with open(fname, 'rb') as f:
+        if not self.admin_url:
+            raise ChrisRequestException(f"User '{self.username}' is not a ChRIS admin.")
+        if isinstance(data, str):
+            with open(data, 'rb') as f:
                 file_contents = f.read()
+        elif type(data) is dict:
+            file_contents = json.dumps(data, indent = 4).encode('utf-8')
         else:
             file_contents = fname.read()
         req = Request(self.username, self.password, self.content_type)
-        data = {'compute_names': compute_names}
-        coll = req.post(self.admin_url, data, file_contents, self.timeout)
+        comp = {'compute_names': compute_names}
+        coll = req.post(self.admin_url, comp, file_contents, self.timeout)
         result = req.get_data_from_collection(coll)
         return result['data'][0]
 
